# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This module document the usage of class pharbers command context,
"""
import io
import sys
import atexit
import logging
from phcli.define_value import *
from phcli.ph_aws.ph_s3 import PhS3

LOG_DEBUG_LEVEL = logging.DEBUG
LOG_INFO_LEVEL = logging.INFO
LOG_WARN_LEVEL = logging.WARNING
LOG_ERROR_LEVEL = logging.ERROR
LOG_DEFAULT_LEVEL = LOG_INFO_LEVEL

LOG_PATH = '{}/logs/python/phcli/{}'

PH_CLI_ACCESS_KEY = 'AKIAWPBDTVEAMVOYSOVE'
PH_CLI_SECRET_KEY = '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'


class PhLogs(object):
    """The Pharbers Logs
    """

    def __init__(self, *args, **kwargs):
        self._log_path = ''
        self.logger = logging.getLogger("ph-log")
        formatter = logging.Formatter("{ 'Time': %(asctime)s, 'Message': %(message)s, 'File': %(filename)s, 'Func': "
                                      "%(funcName)s, 'Line': %(lineno)s, 'Level': %(levelname)s } ")

        sys_handler = logging.StreamHandler(stream=sys.stdout)
        sys_handler.setFormatter(formatter)
        for handler in self.logger.handlers:
            self.logger.removeHandler(handler)
        self.logger.addHandler(sys_handler)

        if 'job_id' in kwargs.keys() and kwargs['job_id']:
            self._log_path = LOG_PATH.format(CLI_VERSION, kwargs['job_id'] + '.log')

        def write_s3_logs(body, bucket, key):
            phs3 = PhS3(access_key=PH_CLI_ACCESS_KEY, secret_key=PH_CLI_SECRET_KEY)
            phs3.s3_client.put_object(Body=body.getvalue(), Bucket=bucket, Key=key)

        if self._log_path and 'storage' in kwargs.keys() and kwargs['storage'] == 's3':
            log_stream = io.StringIO()
            io_handler = logging.StreamHandler(log_stream)
            io_handler.setFormatter(formatter)
            self.logger.addHandler(io_handler)
            atexit.register(write_s3_logs, body=log_stream, bucket=CLI_BUCKET, key=self._log_path)


inst_map = {}


def phs3logger(job_id=None, level=LOG_DEFAULT_LEVEL):
    if job_id and job_id in inst_map.keys():
        return inst_map[job_id]

    logger = PhLogs(job_id=job_id, storage='s3').logger
    logger.setLevel(level)
    inst_map[job_id] = logger

    return logger


phlogger = phs3logger()


if __name__ == '__main__':
    phlogger.debug('debug')
    phlogger.info('info')
    phlogger.warning('warning')
    phlogger.error('error')
    phlogger.critical('critical')

    nulllog = phs3logger('')
    nulllog.debug('debug')
    nulllog.info('info')
    nulllog.warning('warning')
    nulllog.error('error')
    nulllog.critical('critical')

    joblog = phs3logger('job')
    joblog.debug('debug')
    joblog.info('info')
    joblog.warning('warning')
    joblog.error('error')
    joblog.critical('critical')




from phcli.ph_aws.ph_sts import PhSts
from phcli.ph_aws.ph_s3 import PhS3
from phcli.ph_storage.model.local_storage import PhLocalStorage
from phcli.ph_storage.model.hdfs_storage import PhHdfsStorage

phsts = PhSts()
phsts = phsts.assume_role('arn:aws-cn:iam::444603803904:role/Ph-Cli-Lmd', 'Ph-Cli-Lmd')

phs3 = PhS3(phsts=phsts)


def test_ph_clean_logs_remove_local_file():
    ins = PhLocalStorage()
    assert ins.remove("/Users/qianpeng/Desktop/test.txt") is True


def test_ph_clean_logs_exec():
    from phcli.ph_storage.model.local_storage import PhLocalStorage
    from phcli.ph_storage.model.s3_storage import PhS3Storage
    ins = PhHdfsStorage(PhLocalStorage(), PhS3Storage())
    assert ins.back_up('["hdfs://backup:8020/common/public/cpa/0.0.4/YEAR=2015.0/MONTH=2.0"]') is True



# -*- coding: utf-8 -*-

import os
import yaml
from phcli.ph_errs.ph_err import PhException


def load_by_file(file):
    """
    从文件加载 yaml 文件
    :return: obj 对象实体
    """
    with open(file, encoding='UTF-8') as file:
        return yaml.load(file, Loader=yaml.FullLoader)


def append_to_file(obj, file):
    """
    追加写入 yaml 文件
    """
    with open(file, 'a', encoding='UTF-8') as file:
        yaml.dump(obj, file, default_flow_style=False, encoding='utf-8', allow_unicode=True)


def override_to_file(obj, file):
    """
    覆盖写入 yaml 文件
    """
    with open(file, 'w', encoding='UTF-8') as file:
        yaml.dump(obj, file, default_flow_style=False, encoding='utf-8', allow_unicode=True)


def load_by_dir(dir):
    """
    从目录加载多个 yaml 文件
    :return: list[obj] 对象实体列表
    """
    # 参数不是目录，直接返回
    if not os.path.isdir(dir):
        raise PhException("args not is dir")

    if not dir.endswith("/"):
        dir = dir + "/"

    lst = []
    for sub in os.listdir(dir):
        file = dir + sub
        if not (os.path.isfile(file) or file.endswith('.yaml')):
            continue

        obj = load_by_file(file)
        if isinstance(obj, list):
            lst += obj
        else:
            lst.append(obj)

    return lst
